# ! escape values for the announce format
#   album      the album the current song is from
#   artist     the artist of the current song     
#   artists    artists' names separated by " | " otherwise the same as !artist
#   bitrate    bit rate of the current song or just "VBR" for variable bit rate
#              files
#   by         the same as "by !artist" or empty if the artist it not known
#   by_all     the same as "by [!artists]" or empty if the artist is not known
#   duration   duration of the file in MM:SS format
#   duration_s duration of the file in seconds
#   file_size  size of the file "4 MB",
#   format     format of the item being played, such as mp3, wma, avi
#   from       same as "from !album" except it will be empty if the album name
#              is not known/filled
#   percent    percent of the file played so far
#   position   the current position of the file in MM:SS format
#   position_s the current position of the file in seconds,
#   title      name of the current song or video
#   track      the track number
#   track_num  the same as "track !track" or empty if the track is not known
#   type => "audio",
#   version    the WMP version, 9, 10, 11, ...

my $ANNOUNCE_FORMAT = "np: !title !by !from [!position / !duration]";
my $ANNOUNCE_AS_ACTION = 0;

# multiple spaces will be collapsed into a single space
my $SQUASH_ADJACENT_SPACES = 1;

use strict;
use warnings;
use Xchat qw(:all);
use Win32::OLE qw(CP_UTF8);
use Data::Dumper();

# references
# Interfaces
# http://msdn.microsoft.com/en-us/library/dd563068%28v=VS.85%29.aspx
#
# IWMPCore
# http://msdn.microsoft.com/en-us/library/dd563216%28v=VS.85%29.aspx
#
# IWMPControls
# http://msdn.microsoft.com/en-us/library/dd563179%28v=VS.85%29.aspx
#
# IWMPMedia
# http://msdn.microsoft.com/en-us/library/dd563435%28v=VS.85%29.aspx
# 
# Attributes
# http://msdn.microsoft.com/en-us/library/dd562375%28v=VS.85%29.aspx
# http://msdn.microsoft.com/en-us/library/dd562379%28v=VS.85%29.aspx
#
#

sub DEBUG () { 0 }
Win32::OLE->Initialize;
Win32::OLE->Option( Warn => DEBUG ? 2 : 0, CP => CP_UTF8 );

my $NAME = "WMP Announce";
register(
	$NAME,
	"1.000000",
	"Display what WMP is playing",
	sub { prnt "$NAME unloaded" }
);

hook_command( "WMP", \&cmd_wmp, { help_text => usage_text() } );

my %action_map = (
	np => \&announce,
	announce => \&announce,
	play => \&play,
	stop => \&stop,
	pause => \&pause,
	next => \&next,
	previous => \&previous,
	debug => sub { Dwarn( get_fields() ) },
);

sub cmd_wmp {
	my ( undef, $action ) = @{$_[0]};
	$action = lc $action;
	$action ||= 'announce';

	if( my $code = $action_map{ $action } ) {
		eval {
			$code->( @_ );
		};

		if( $@ ) {
			prnt "Error while trying to perform action '$action'";
			prnt $@;
		}
	} else {
		prnt "'$action' is not a valid action for $_[0][0]";
		usage();
	}

	return EAT_XCHAT;
}

sub usage {
	prnt usage_text();
}

sub usage_text {
	return <<'USAGE';
WMP [action]
   Possible actions
      np, announce  display the currently playing song in the channel
         Options for announce:
         -a         annouce as an action
         format     format to use instead of the one configured in the script

      play          start playing
      stop          stop playing
      pause         pause wmp
      next          skip to the next song
      previous      go back to the previous song
      usage         show this message

   If not providing an action is the same as using announce.
   The following are equivalent.

   /WMP
   /WMP np
   /WMP announce
USAGE
}

{ my $wmp;
	sub wmp {
		unless( $wmp ) {
			$wmp = (
				Win32::OLE->new( 'WMPuICE.WMPApp' )
				|| Win32::OLE->new( 'WMPuICE.WMPApp.1' )
				|| Win32::OLE->new( '{2667A030-07F5-45F3-B064-8ACA073B1F94}' )
			) or die Win32::OLE->LastError;
		}
		return $wmp;
	}
}

# IWMPCore
sub core {
	return wmp->{Core};
}

# IWMPControls
sub controls {
	return core->controls;
}

# IWMPMedia
sub media {
	return core->currentMedia;
}

sub wmp_running {
	wmp()->{Running};
}

#typedef enum WMPPlayState {
#  wmppsUndefined       = 0,
#  wmppsStopped         = 1,
#  wmppsPaused          = 2,
#  wmppsPlaying         = 3,
#  wmppsScanForward     = 4,
#  wmppsScanReverse     = 5,
#  wmppsBuffering       = 6,
#  wmppsWaiting         = 7,
#  wmppsMediaEnded      = 8,
#  wmppsTransitioning   = 9,
#  wmppsReady           = 10,
#  wmppsReconnecting    = 11,
#  wmppsLast            = 12 
#} WMPPlayState;
#
sub wmp_playing {
	warn "Current play state is " . core->playState if DEBUG;
	return core->playState == 3;
}

sub play { controls->play; }
sub stop { controls->stop; }
sub pause { controls->pause; }
sub next { controls->next; }
sub previous { controls->previous; }

sub announce {

	unless( wmp_running ) {
		prnt "WMP is not currently running";
		return EAT_XCHAT;
	}

	unless( wmp_playing ) {
		prnt "WMP is not currently playing anything";
		return EAT_XCHAT;
	}

	my $as_action;
	my $template;
	
	if( $_[0][2] and $_[0][2] eq '-a' ) {
		$as_action = 1;
		$template = $_[1][3];
	} else {
		$as_action = $ANNOUNCE_AS_ACTION;
		$template = $_[1][2];
	}

	$template ||= $ANNOUNCE_FORMAT;
	my $message = render_message( $template );

	if( $SQUASH_ADJACENT_SPACES ) {
		$message =~ tr/ / /s;
	}

	if( $as_action ) {
		command( "ME $message" );
	} else {
		command( "SAY $message" );
	}

	return EAT_XCHAT;
}

sub render_message {
	my $message = shift;

	my $fields = get_fields();
	my $pattern = join( '|', '!', keys %$fields );
	my %stash = ( %$fields, '!' => '!' );

	$message =~ s/!($pattern)/$stash{$1}/g;

	return $message;
}

sub get_fields {
	my $media = media;
	my $control = controls;

	my %fields = (
		title => $media->name,
		artist => $media->getItemInfo( "Author" ), # Primary Actor, Artist
		artists => do { # All author/actor/artist seperated by |
			my $attr_count = $media->getAttributeCountByType( "Author", "" );
			my $authors;

			if( $attr_count > 1 ) {
				my @authors =
					map { $media->getItemInfoByType( "Author", "", $_ ) }
						0 .. $attr_count - 1;
				
				$authors = join " | ", @authors;
			} else {
				$authors = $media->getItemInfo( "Author" );
			}
			$authors;
		},
#		album_artist => $media->getItemInfo( "AlbumArtist" ) # "WM/AlbumArtist"
		album => $media->getItemInfo( "Album" ), # "WM/AlbumTitle"
		position => $control->currentPositionString,
		position_s => $control->currentPosition,
		duration => $media->durationString,
		duration_s => $media->duration,
		percent => int( $control->currentPosition / $media->duration * 100 ),
		bitrate => do {
			my $bitrate;
			my $is_vbr = $media->getItemInfo( "IsVBR" );
			
			if( $is_vbr =~ /true/i ) {
				$bitrate = "VBR";
			} else {
				$bitrate = $media->getItemInfo( "Bitrate" );
				$bitrate = format_bits( $bitrate ) . "ps";
			}
			$bitrate;
		},
		track => $media->getItemInfo( "OriginalIndex" ), # "WM/TrackNumber"
		type => $media->getItemInfo( "MediaType" ), # audio, video, etc.
		format => $media->getItemInfo( "FileType" ), # mp3, wma, wmv, etc.
		file_size => format_bytes( $media->getItemInfo( "FileSize" ) ),
		version => do {
			my ($version) = core->versionInfo =~ /^(\d+)/;
			$version;
		},
	);

	$fields{by} = do {
		my $artist = $fields{artist} || $media->getItemInfo( "AlbumArtist" );
		$artist ? "by $artist" : "";
	};

	$fields{by_all} = do {
		my $artists = $fields{artists} || $media->getItemInfo( "AlbumArtist" );
		$artists ? "by [$artists]" : "";
	};

	$fields{from} = do {
		my $album = $fields{album};
		$album ? "from $album" : "";
	};

	$fields{track_num} = do {
		my $track_num = $fields{track};
		$track_num ? "track $track_num" : "";
	};

	return \%fields;
}

sub Dwarn {
	local $Data::Dumper::Terse = 1;
	local $Data::Dumper::Indent = 1;
	local $Data::Dumper::Useqq = 1;
	local $Data::Dumper::Deparse = 1;
	local $Data::Dumper::Quotekeys = 0;
	local $Data::Dumper::Sortkeys = 1;
	warn Data::Dumper::Dumper( @_ );
}

sub format_bits {
	return format_number( shift ) . "b";
}

sub format_bytes {
	return format_number( shift ) . "B";
}

sub format_number {
   my $size = shift;

	return "" unless $size;
   if( $size >= 1000**3 ) { # G
      return sprintf "%.0f G", $size/1000**3;
   } elsif( $size >= 1000**2 ) { # M
      return sprintf "%.0f M", $size/1000**2;
   } elsif( $size >= 1000 ) { # K
      return sprintf "%.0f K", $size/1000;
   } else {
      return sprintf "%.0f ", $size;
   }

}

prnt "$NAME loaded";
